@@ -4,13 +4,14 @@ reV aggregation framework.
 """
 from abc import ABC, abstractmethod, abstractstaticmethod
 from concurrent.futures import as_completed
-import logging
 import h5py
+import logging
 import numpy as np
 import os
 import pandas as pd
 
 from reV.handlers.resource import Resource
+from reV.handlers.outputs import Outputs
 from reV.supply_curve.exclusions import ExclusionMaskFromDict
 from reV.supply_curve.points import (SupplyCurveExtent,
                                      AggregationSupplyCurvePoint)
@@ -202,7 +203,7 @@ class AbstractAggregation(ABC):
     def run_serial(sc_point_method, excl_fpath, tm_dset,
                    excl_dict=None, area_filter_kernel='queen',
                    min_area=None, check_excl_layers=False,
-                   resolution=64, gids=None, close=False, args=None,
+                   resolution=64, gids=None, args=None,
                    kwargs=None):
         """Standalone method to create agg summary - can be parallelized.
 
@@ -230,8 +231,6 @@ class AbstractAggregation(ABC):
         gids : list | None
             List of gids to get summary for (can use to subset if running in
             parallel), or None for all gids in the SC extent.
-        close : bool
-            Flag to close object file handlers on exit.
         args : list | None
             List of positional args for sc_point_method
         kwargs : dict | None
@@ -272,7 +271,7 @@ class AbstractAggregation(ABC):
                         excl_dict=excl_dict,
                         resolution=resolution,
                         exclusion_shape=exclusion_shape,
-                        close=close,
+                        close=False,
                         **kwargs)
 
                 except EmptySupplyCurvePointError:
@@ -285,7 +284,7 @@ class AbstractAggregation(ABC):
 
     @abstractmethod
     def run_parallel(self, sc_point_method, args=None, kwargs=None,
-                     close=False, max_workers=None, chunk_point_len=1000):
+                     max_workers=None, chunk_point_len=1000):
         """
         Aggregate with sc_point_method in parallel
 
@@ -295,8 +294,6 @@ class AbstractAggregation(ABC):
             List of positional args for sc_point_method
         kwargs : dict | None
             Dict of kwargs for sc_point_method
-        close : bool
-            Flag to close object file handlers on exit.
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
@@ -336,7 +333,6 @@ class AbstractAggregation(ABC):
                     check_excl_layers=self._check_excl_layers,
                     resolution=self._resolution,
                     gids=gid_set,
-                    close=close,
                     args=args,
                     kwargs=kwargs))
 
@@ -351,7 +347,7 @@ class AbstractAggregation(ABC):
         return output
 
     def aggregate(self, sc_point_method, args=None, kwargs=None,
-                  close=False, max_workers=None, chunk_point_len=1000):
+                  max_workers=None, chunk_point_len=1000):
         """
         Aggregate with sc_point_method
 
@@ -361,8 +357,6 @@ class AbstractAggregation(ABC):
             List of positional args for sc_point_method
         kwargs : dict | None
             Dict of kwargs for sc_point_method
-        close : bool
-            Flag to close object file handlers on exit.
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
@@ -386,12 +380,11 @@ class AbstractAggregation(ABC):
                                   check_excl_layers=self._check_excl_layers,
                                   resolution=self._resolution,
                                   gids=self._gids,
-                                  close=close, args=args,
+                                  args=args,
                                   kwargs=kwargs)
         else:
             agg = self.run_parallel(sc_point_method, args=args,
-                                    kwargs=kwargs, close=close,
-                                    max_workers=max_workers,
+                                    kwargs=kwargs, max_workers=max_workers,
                                     chunk_point_len=chunk_point_len)
 
         if not any(agg):
@@ -406,8 +399,7 @@ class AbstractAggregation(ABC):
     def run(cls, excl_fpath, tm_dset, sc_point_method, excl_dict=None,
             area_filter_kernel='queen', min_area=None,
             check_excl_layers=False, resolution=64, gids=None,
-            args=None, kwargs=None, close=False, max_workers=None,
-            chunk_point_len=1000):
+            args=None, kwargs=None, max_workers=None, chunk_point_len=1000):
         """Get the supply curve points aggregation summary.
 
         Parameters
@@ -440,8 +432,6 @@ class AbstractAggregation(ABC):
             List of positional args for sc_point_method
         kwargs : dict | None
             Dict of kwargs for sc_point_method
-        close : bool
-            Flag to close object file handlers on exit.
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
@@ -460,7 +450,7 @@ class AbstractAggregation(ABC):
                   gids=gids)
 
         aggregation = agg.aggregate(sc_point_method, args=args, kwargs=kwargs,
-                                    close=close, max_workers=max_workers,
+                                    max_workers=max_workers,
                                     chunk_point_len=chunk_point_len)
 
         return aggregation
@@ -497,8 +487,8 @@ class Aggregation(AbstractAggregation):
             SC resolution, must be input in combination with gid. Prefered
             option is to use the row/col slices to define the SC point instead.
         gids : list | None
-            List of gids to get summary for (can use to subset if running in
-            parallel), or None for all gids in the SC extent.
+            List of gids to get aggregation for (can use to subset if running
+            in parallel), or None for all gids in the SC extent.
         """
         super().__init__(excl_fpath, tm_dset, excl_dict=excl_dict,
                          area_filter_kernel=area_filter_kernel,
@@ -513,6 +503,7 @@ class Aggregation(AbstractAggregation):
         self._agg_dsets = agg_dset
 
         self._check_files()
+        self._gen_index = self._parse_gen_index(self._h5_fpath)
 
     def _check_files(self):
         """Do a preflight check on input files"""
@@ -532,19 +523,51 @@ class Aggregation(AbstractAggregation):
                                      .format(self._tm_dset,
                                              self._excl_fpath))
 
-        with h5py.File(self._excl_fpath, 'r') as f:
+        with Resource(self._h5_fpath) as f:
             for dset in self._agg_dsets:
                 if dset not in f:
                     raise FileInputError('Could not find provided dataset "{}"'
                                          ' in h5 file: {}'
                                          .format(dset, self._h5_fpath))
 
+    @staticmethod
+    def _parse_gen_index(h5_fpath):
+        """Parse gen outputs for an array of generation gids corresponding to
+        the resource gids.
+
+        Parameters
+        ----------
+        h5_fpath : str
+            Filepath to reV compliant .h5 file
+
+        Returns
+        -------
+        gen_index : np.ndarray
+            Array of generation gids with array index equal to resource gid.
+            Array value is -1 if the resource index was not used in the
+            generation run.
+        """
+
+        with Resource(h5_fpath) as f:
+            gen_index = f.meta
+
+        gen_index = gen_index.rename(columns={'gid': 'res_gids'})
+        gen_index['gen_gids'] = gen_index.index
+        gen_index = gen_index[['res_gids', 'gen_gids']]
+        gen_index = gen_index.set_index(keys='res_gids')
+        gen_index = gen_index.reindex(range(int(gen_index.index.max() + 1)))
+        gen_index = gen_index['gen_gids'].values
+        gen_index[np.isnan(gen_index)] = -1
+        gen_index = gen_index.astype(np.int32)
+
+        return gen_index
+
     @staticmethod
     def run_serial(excl_fpath, h5_fpath, tm_dset, *agg_dset,
                    agg_method='mean', excl_dict=None,
                    area_filter_kernel='queen', min_area=None,
                    check_excl_layers=False, resolution=64, excl_area=0.0081,
-                   gids=None, close=False):
+                   gids=None, gen_index=None):
         """
         Standalone method to aggregate - can be parallelized.
 
@@ -573,11 +596,15 @@ class Aggregation(AbstractAggregation):
         resolution : int | None
             SC resolution, must be input in combination with gid. Prefered
             option is to use the row/col slices to define the SC point instead.
+        excl_area : float
+            Area of an exclusion cell (square km).
         gids : list | None
             List of gids to get summary for (can use to subset if running in
             parallel), or None for all gids in the SC extent.
-        close : bool
-            Flag to close object file handlers on exit.
+        gen_index : np.ndarray
+            Array of generation gids with array index equal to resource gid.
+            Array value is -1 if the resource index was not used in the
+            generation run.
 
         Returns
         -------
@@ -594,10 +621,12 @@ class Aggregation(AbstractAggregation):
                        'area_filter_kernel': area_filter_kernel,
                        'min_area': min_area,
                        'check_excl_layers': check_excl_layers}
+        dsets = agg_dset + ('meta', )
+        agg_out = {ds: [] for ds in dsets}
         with AggFileHandler(excl_fpath, h5_fpath, **file_kwargs) as fh:
             for gid in gids:
                 try:
-                    gid_out = AggregationSupplyCurvePoint(
+                    gid_out = AggregationSupplyCurvePoint.run(
                         gid,
                         fh.exclusions,
                         fh.h5,
@@ -608,14 +637,19 @@ class Aggregation(AbstractAggregation):
                         resolution=resolution,
                         excl_area=excl_area,
                         exclusion_shape=exclusion_shape,
-                        close=close)
+                        close=False,
+                        gen_index=gen_index)
 
                 except EmptySupplyCurvePointError:
-                    pass
+                    gid_out = None
+                else:
+                    if gid_out is not None:
+                        for k, v in gid_out.items():
+                            agg_out[k].append(v)
 
-        return gid_out
+        return agg_out
 
-    def run_parallel(self, agg_method='mean', excl_area=0.0081, close=False,
+    def run_parallel(self, agg_method='mean', excl_area=0.0081,
                      max_workers=None, chunk_point_len=1000):
         """
         Aggregate in parallel
@@ -624,8 +658,8 @@ class Aggregation(AbstractAggregation):
         ----------
         agg_method : str
             Aggregation method, either mean or sum/aggregate
-        close : bool
-            Flag to close object file handlers on exit.
+        excl_area : float
+            Area of an exclusion cell (square km).
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
@@ -668,7 +702,7 @@ class Aggregation(AbstractAggregation):
                     resolution=self._resolution,
                     excl_area=excl_area,
                     gids=gid_set,
-                    close=close))
+                    gen_index=self._gen_index))
 
             # gather results
             for future in futures:
@@ -676,26 +710,13 @@ class Aggregation(AbstractAggregation):
                 logger.info('Parallel aggregation futures collected: '
                             '{} out of {}'
                             .format(n_finished, len(chunks)))
-                for k, v in future.results():
-                    agg_out[k].append(v)
-
-        for k, v in agg_out.items():
-            if k == 'meta':
-                v = pd.concat(v, axis=1).T
-                v = v.sort_values('sc_point_gid')
-                v = v.reset_index(drop=True)
-                v.index.name = 'sc_gid'
-                agg_out[k] = v
-            else:
-                v = np.dstack(v)[0]
-                if v.shape[0] == 1:
-                    v = v.flatten()
-
-                agg_out[k] = v
+                for k, v in future.result().items():
+                    if v:
+                        agg_out[k].extend(v)
 
         return agg_out
 
-    def aggregate(self, agg_method='mean', excl_area=0.0081, close=False,
+    def aggregate(self, agg_method='mean', excl_area=0.0081,
                   max_workers=None, chunk_point_len=1000):
         """
         Aggregate with given agg_method
@@ -704,8 +725,8 @@ class Aggregation(AbstractAggregation):
         ----------
         agg_method : str
             Aggregation method, either mean or sum/aggregate
-        close : bool
-            Flag to close object file handlers on exit.
+        excl_area : float
+            Area of an exclusion cell (square km).
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
@@ -732,37 +753,100 @@ class Aggregation(AbstractAggregation):
                                   check_excl_layers=self._check_excl_layers,
                                   resolution=self._resolution,
                                   excl_area=excl_area,
-                                  close=close)
+                                  gen_index=self._gen_index)
         else:
             agg = self.run_parallel(agg_method=agg_method, excl_area=excl_area,
-                                    close=close, max_workers=max_workers,
+                                    max_workers=max_workers,
                                     chunk_point_len=chunk_point_len)
 
-        if not any(agg['meta']):
+        if not agg['meta']:
             e = ('Supply curve aggregation found no non-excluded SC points. '
                  'Please check your exclusions or subset SC GID selection.')
             logger.error(e)
             raise EmptySupplyCurvePointError(e)
 
+        for k, v in agg.items():
+            if k == 'meta':
+                v = pd.concat(v, axis=1).T
+                v = v.sort_values('sc_point_gid')
+                v = v.reset_index(drop=True)
+                v.index.name = 'sc_gid'
+                agg[k] = v.reset_index()
+            else:
+                v = np.dstack(v)[0]
+                if v.shape[0] == 1:
+                    v = v.flatten()
+
+                agg[k] = v
+
         return agg
 
+    def save_agg_to_h5(self, out_fpath, aggregation):
+        """
+        Save aggregated data to disc in .h5 format
+
+        Parameters
+        ----------
+        out_fpath : str
+            Output .h5 file path
+        aggregation : dict
+            Aggregated values for each aggregation dataset
+        """
+        agg_out = aggregation.copy()
+        meta = agg_out.pop('meta')
+        for c in meta.columns:
+            try:
+                meta[c] = pd.to_numeric(meta[c])
+            except (ValueError, TypeError):
+                pass
+
+        dsets = []
+        shapes = {}
+        attrs = {}
+        chunks = {}
+        dtypes = {}
+        time_index = None
+        with Resource(self._h5_fpath) as f:
+            for dset, data in agg_out.items():
+                dsets.append(dset)
+                shape = data.shape
+                shapes[dset] = shape
+                if len(data.shape) == 2:
+                    if ('time_index' in f) and (shape[0] == f.shape[0]):
+                        if time_index is None:
+                            time_index = f.time_index
+
+                attrs[dset] = f.get_attrs(dset=dset)
+                _, dtype, chunk = f.get_dset_properties(dset)
+                chunks[dset] = chunk
+                dtypes[dset] = dtype
+
+        Outputs.init_h5(out_fpath, dsets, shapes, attrs, chunks, dtypes,
+                        meta, time_index=time_index)
+
+        with Outputs(out_fpath, mode='a') as out:
+            for dset, data in agg_out.items():
+                out[dset] = data
+
     @classmethod
-    def run(cls, excl_fpath, tm_dset, sc_point_method, excl_dict=None,
-            area_filter_kernel='queen', min_area=None,
+    def run(cls, excl_fpath, h5_fpath, tm_dset, *agg_dset,
+            excl_dict=None, area_filter_kernel='queen', min_area=None,
             check_excl_layers=False, resolution=64, gids=None,
-            args=None, kwargs=None, close=False, max_workers=None,
-            chunk_point_len=1000):
+            agg_method='mean', excl_area=0.0081, max_workers=None,
+            chunk_point_len=1000, out_fpath=None):
         """Get the supply curve points aggregation summary.
 
         Parameters
         ----------
         excl_fpath : str
             Filepath to exclusions h5 with techmap dataset.
+        h5_fpath : str
+            Filepath to .h5 file to aggregate
         tm_dset : str
             Dataset name in the techmap file containing the
             exclusions-to-resource mapping data.
-        sc_point_method : method
-            Supply Curve Point Method to operate on a single SC point.
+        agg_dset : str
+            Dataset to aggreate, can supply multiple datasets
         excl_dict : dict | None
             Dictionary of exclusion LayerMask arugments {layer: {kwarg: value}}
         area_filter_kernel : str
@@ -776,35 +860,36 @@ class Aggregation(AbstractAggregation):
             SC resolution, must be input in combination with gid. Prefered
             option is to use the row/col slices to define the SC point instead.
         gids : list | None
-            List of gids to get summary for (can use to subset if running in
-            parallel), or None for all gids in the SC extent.
-        sc_point_method : method
-            Supply Curve Point Method to operate on a single SC point.
-        args : list | None
-            List of positional args for sc_point_method
-        kwargs : dict | None
-            Dict of kwargs for sc_point_method
-        close : bool
-            Flag to close object file handlers on exit.
+            List of gids to get aggregation for (can use to subset if running
+            in parallel), or None for all gids in the SC extent.
+        agg_method : str
+            Aggregation method, either mean or sum/aggregate
+        excl_area : float
+            Area of an exclusion cell (square km).
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
         chunk_point_len : int
             Number of SC points to process on a single parallel worker.
+        out_fpath : str
+            Output .h5 file path
 
         Returns
         -------
-        summary : DataFrame
-            Summary of the SC points.
+        agg : dict
+            Aggregated values for each aggregation dataset
         """
 
-        agg = cls(excl_fpath, tm_dset, excl_dict=excl_dict,
-                  area_filter_kernel=area_filter_kernel, min_area=min_area,
-                  check_excl_layers=check_excl_layers, resolution=resolution,
-                  gids=gids)
+        agg = cls(excl_fpath, h5_fpath, tm_dset, *agg_dset,
+                  excl_dict=excl_dict, area_filter_kernel=area_filter_kernel,
+                  min_area=min_area, check_excl_layers=check_excl_layers,
+                  resolution=resolution, gids=gids)
 
-        aggregation = agg.aggregate(sc_point_method, args=args, kwargs=kwargs,
-                                    close=close, max_workers=max_workers,
+        aggregation = agg.aggregate(agg_method=agg_method, excl_area=excl_area,
+                                    max_workers=max_workers,
                                     chunk_point_len=chunk_point_len)
 
+        if out_fpath is not None:
+            agg.save_agg_to_h5(out_fpath, aggregation)
+
         return aggregation
