@@ -17,7 +17,8 @@ from warnings import warn
 from reV.handlers.exclusions import ExclusionLayers
 from reV.handlers.resource import Resource
 from reV.supply_curve.aggregation import (AbstractAggFileHandler,
-                                          AbstractAggregation)
+                                          AbstractAggregation,
+                                          Aggregation)
 from reV.supply_curve.exclusions import FrictionMask
 from reV.supply_curve.points import SupplyCurveExtent
 from reV.supply_curve.point_summary import SupplyCurvePointSummary
@@ -302,7 +303,7 @@ class SupplyCurveAggregation(AbstractAggregation):
             warn(msg, InputWarning)
 
         self._check_data_layers()
-        self._gen_index = self._parse_gen_index(self._gen_fpath)
+        self._gen_index = Aggregation._parse_gen_index(self._gen_fpath)
 
     def _check_files(self):
         """Do a preflight check on input files"""
@@ -352,38 +353,6 @@ class SupplyCurveAggregation(AbstractAggregation):
                                    .format(k, f.shape, shape_base))
                             raise FileInputError(msg)
 
-    @staticmethod
-    def _parse_gen_index(gen_fpath):
-        """Parse gen outputs for an array of generation gids corresponding to
-        the resource gids.
-
-        Parameters
-        ----------
-        gen_fpath : str
-            Filepath to .h5 reV generation output results.
-
-        Returns
-        -------
-        gen_index : np.ndarray
-            Array of generation gids with array index equal to resource gid.
-            Array value is -1 if the resource index was not used in the
-            generation run.
-        """
-
-        with Resource(gen_fpath) as gen:
-            gen_index = gen.meta
-
-        gen_index = gen_index.rename(columns={'gid': 'res_gids'})
-        gen_index['gen_gids'] = gen_index.index
-        gen_index = gen_index[['res_gids', 'gen_gids']]
-        gen_index = gen_index.set_index(keys='res_gids')
-        gen_index = gen_index.reindex(range(int(gen_index.index.max() + 1)))
-        gen_index = gen_index['gen_gids'].values
-        gen_index[np.isnan(gen_index)] = -1
-        gen_index = gen_index.astype(np.int32)
-
-        return gen_index
-
     @staticmethod
     def _get_input_data(gen, gen_fpath, res_class_dset, res_class_bins,
                         cf_dset, lcoe_dset):
@@ -461,11 +430,10 @@ class SupplyCurveAggregation(AbstractAggregation):
     def run_serial(excl_fpath, gen_fpath, tm_dset, gen_index,
                    excl_dict=None, area_filter_kernel='queen', min_area=None,
                    check_excl_layers=False, resolution=64, gids=None,
-                   close=False, args=None, res_class_dset=None,
-                   res_class_bins=None, cf_dset='cf_mean-means',
-                   lcoe_dset='lcoe_fcr-means', data_layers=None,
-                   power_density=None, friction_fpath=None, friction_dset=None,
-                   excl_area=0.0081):
+                   args=None, res_class_dset=None, res_class_bins=None,
+                   cf_dset='cf_mean-means', lcoe_dset='lcoe_fcr-means',
+                   data_layers=None, power_density=None, friction_fpath=None,
+                   friction_dset=None, excl_area=0.0081):
         """Standalone method to create agg summary - can be parallelized.
 
         Parameters
@@ -496,8 +464,6 @@ class SupplyCurveAggregation(AbstractAggregation):
         gids : list | None
             List of gids to get summary for (can use to subset if running in
             parallel), or None for all gids in the SC extent.
-        close : bool
-            Flag to close object file handlers on exit.
         args : list | None
             List of positional args for sc_point_method
         res_class_dset : str | None
@@ -580,7 +546,7 @@ class SupplyCurveAggregation(AbstractAggregation):
                             args=args,
                             excl_dict=excl_dict,
                             excl_area=excl_area,
-                            close=close,
+                            close=False,
                             offshore_flags=inputs[4],
                             friction_layer=fh.friction_layer)
 
@@ -597,8 +563,7 @@ class SupplyCurveAggregation(AbstractAggregation):
 
         return summary
 
-    def run_parallel(self, args=None, excl_area=0.0081, close=False,
-                     max_workers=None):
+    def run_parallel(self, args=None, excl_area=0.0081, max_workers=None):
         """Get the supply curve points aggregation summary using futures.
 
         Parameters
@@ -608,8 +573,6 @@ class SupplyCurveAggregation(AbstractAggregation):
             available args defined in the class attr.
         excl_area : float
             Area of an exclusion cell (square km).
-        close : bool
-            Flag to close object file handlers on exit.
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
@@ -653,7 +616,7 @@ class SupplyCurveAggregation(AbstractAggregation):
                     friction_dset=self._friction_dset,
                     area_filter_kernel=self._area_filter_kernel,
                     min_area=self._min_area,
-                    gids=gid_set, args=args, excl_area=excl_area, close=close,
+                    gids=gid_set, args=args, excl_area=excl_area,
                     check_excl_layers=self._check_excl_layers))
 
             # gather results
@@ -886,9 +849,9 @@ class SupplyCurveAggregation(AbstractAggregation):
 
         return summary
 
-    def summarize(self, args=None, excl_area=0.0081, close=False,
-                  max_workers=None, offshore_capacity=600,
-                  offshore_gid_counts=494, offshore_pixel_area=4):
+    def summarize(self, args=None, excl_area=0.0081, max_workers=None,
+                  offshore_capacity=600, offshore_gid_counts=494,
+                  offshore_pixel_area=4):
         """
         Get the supply curve points aggregation summary
 
@@ -899,8 +862,6 @@ class SupplyCurveAggregation(AbstractAggregation):
             available args defined in the class attr.
         excl_area : float
             Area of an exclusion cell (square km).
-        close : bool
-            Flag to close object file handlers on exit.
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
@@ -935,11 +896,11 @@ class SupplyCurveAggregation(AbstractAggregation):
                                       area_filter_kernel=afk,
                                       min_area=self._min_area,
                                       gids=self._gids, args=args,
-                                      excl_area=excl_area, close=close,
+                                      excl_area=excl_area,
                                       check_excl_layers=chk)
         else:
             summary = self.run_parallel(args=args, excl_area=excl_area,
-                                        close=close, max_workers=max_workers)
+                                        max_workers=max_workers)
 
         summary = self.run_offshore(summary,
                                     offshore_capacity=offshore_capacity,
@@ -965,9 +926,9 @@ class SupplyCurveAggregation(AbstractAggregation):
                 cf_dset='cf_mean-means', lcoe_dset='lcoe_fcr-means',
                 data_layers=None, power_density=None,
                 friction_fpath=None, friction_dset=None,
-                args=None, excl_area=0.0081, close=False,
-                max_workers=None, offshore_capacity=600,
-                offshore_gid_counts=494, offshore_pixel_area=4):
+                args=None, excl_area=0.0081, max_workers=None,
+                offshore_capacity=600, offshore_gid_counts=494,
+                offshore_pixel_area=4):
         """Get the supply curve points aggregation summary.
 
         Parameters
@@ -1024,8 +985,6 @@ class SupplyCurveAggregation(AbstractAggregation):
             available args defined in the class attr.
         excl_area : float
             Area of an exclusion cell (square km).
-        close : bool
-            Flag to close object file handlers on exit.
         max_workers : int | None
             Number of cores to run summary on. None is all
             available cpus.
@@ -1052,7 +1011,7 @@ class SupplyCurveAggregation(AbstractAggregation):
                   area_filter_kernel=area_filter_kernel, min_area=min_area,
                   check_excl_layers=check_excl_layers)
 
-        summary = agg.summarize(args=args, excl_area=excl_area, close=close,
+        summary = agg.summarize(args=args, excl_area=excl_area,
                                 max_workers=max_workers,
                                 offshore_capacity=offshore_capacity,
                                 offshore_gid_counts=offshore_gid_counts,
